@@ -14,9 +14,8 @@
 #include "torrent/object.h"
 #include "torrent/object_static_map.h"
 #include "torrent/object_stream.h"
-#include "torrent/poll.h"
-#include "torrent/throttle.h"
 #include "torrent/net/fd.h"
+#include "torrent/net/poll.h"
 #include "torrent/net/socket_address.h"
 #include "torrent/net/network_config.h"
 #include "torrent/net/network_manager.h"
@@ -72,11 +71,9 @@ const DhtMessage::key_list_type DhtMessage::base_type::keys = {
 };
 
 DhtServer::DhtServer(DhtRouter* router)
-  : m_router(router),
-    m_uploadThrottle(manager->upload_throttle()->throttle_list()),
-    m_downloadThrottle(manager->download_throttle()->throttle_list()) {
+  : m_router(router) {
 
-  get_fd().clear();
+  m_fileDesc = -1;
   reset_statistics();
 
   // Reserve a socket for the DHT server, even though we don't
@@ -95,14 +92,28 @@ DhtServer::~DhtServer() {
 
 void
 DhtServer::start(int port) {
+  auto [bind_inet_address, bind_inet6_address] = config::network_config()->bind_addresses_or_null();
+
+  if (bind_inet_address == nullptr)
+    throw resource_error("no valid bind address for DHT server");
+
+  sa_unique_ptr bind_address;
+
+  switch (bind_inet_address->sa_family) {
+  case AF_INET:
+    bind_address = sa_copy(bind_inet_address.get());
+    break;
+  case AF_UNSPEC:
+    bind_address = sa_make_inet_any();
+    break;
+  default:
+    throw resource_error("invalid address family for DHT server");
+  }
+
+  m_router->set_address(bind_inet_address.get());
+  sap_set_port(bind_address, port);
+
   try {
-    auto bind_address = sa_copy(m_router->address());
-
-    if (bind_address->sa_family != AF_INET && bind_address->sa_family != AF_INET6)
-      throw resource_error("invalid address family for DHT server");
-
-    sap_set_port(bind_address, port);
-
     LT_LOG_THIS("starting server : %s", sap_pretty_str(bind_address).c_str());
 
     fd_flags open_flags = fd_flag_datagram | fd_flag_nonblock | fd_flag_reuse_address;
@@ -112,6 +123,9 @@ DhtServer::start(int port) {
 
     m_fileDesc = fd_open(open_flags);
 
+    if (m_fileDesc == -1)
+      throw resource_error("could not open datagram socket : " + std::string(strerror(errno)));
+
     // Figure out how to bind to both inet and inet6.
     if (!fd_bind(m_fileDesc, bind_address.get()))
       throw resource_error("could not bind datagram socket : " + std::string(strerror(errno)));
@@ -122,12 +136,6 @@ DhtServer::start(int port) {
     throw;
   }
 
-  m_uploadNode.set_list_iterator(m_uploadThrottle->end());
-  m_uploadNode.slot_activate() = [this] { receive_throttle_up_activate(); };
-
-  m_downloadNode.set_list_iterator(m_downloadThrottle->end());
-  m_downloadThrottle->insert(&m_downloadNode);
-
   this_thread::poll()->open(this);
   this_thread::poll()->insert_read(this);
   this_thread::poll()->insert_error(this);
@@ -143,10 +151,6 @@ DhtServer::stop() {
   clear_transactions();
 
   this_thread::scheduler()->erase(&m_task_timeout);
-
-  m_uploadThrottle->erase(&m_uploadNode);
-  m_downloadThrottle->erase(&m_downloadNode);
-
   this_thread::poll()->remove_and_close(this);
 
   fd_close(m_fileDesc);
@@ -162,9 +166,6 @@ DhtServer::reset_statistics() {
   m_repliesReceived = 0;
   m_errorsReceived = 0;
   m_errorsCaught = 0;
-
-  m_uploadNode.rate()->set_total(0);
-  m_downloadNode.rate()->set_total(0);
 }
 
 // Ping a node whose ID we know.
@@ -502,12 +503,8 @@ DhtServer::drop_packet(DhtTransactionPacket* packet) {
   m_lowQueue.erase(std::remove_if(m_lowQueue.begin(), m_lowQueue.end(), [packet](auto& p) { return p.get() == packet; }),
                    m_lowQueue.end());
 
-  if (m_highQueue.empty() && m_lowQueue.empty()) {
+  if (m_highQueue.empty() && m_lowQueue.empty())
     this_thread::poll()->remove_write(this);
-
-    if (m_uploadThrottle->is_throttled(&m_uploadNode))
-      m_uploadThrottle->erase(&m_uploadNode);
-  }
 }
 
 void
@@ -676,8 +673,6 @@ DhtServer::clear_transactions() {
 
 void
 DhtServer::event_read() {
-  uint32_t total = 0;
-
   while (true) {
     Object request;
     int type = '?';
@@ -704,8 +699,6 @@ DhtServer::event_read() {
       if (sa->sa_family != AF_INET)
         continue;
 
-      total += read;
-
       // If it's not a valid bencode dictionary at all, it's probably not a DHT
       // packet at all, so we don't throw an error to prevent bounce loops.
       try {
@@ -791,16 +784,11 @@ DhtServer::event_read() {
     }
   }
 
-  m_downloadThrottle->node_used_unthrottled(total);
-  m_downloadNode.rate()->insert(total);
-
   start_write();
 }
 
-bool
-DhtServer::process_queue(packet_queue& queue, uint32_t* quota) {
-  uint32_t used = 0;
-
+void
+DhtServer::process_queue(packet_queue& queue) {
   while (!queue.empty()) {
     auto packet = queue.front();
     DhtTransaction::key_type transactionKey = 0;
@@ -816,11 +804,6 @@ DhtServer::process_queue(packet_queue& queue, uint32_t* quota) {
       continue;
     }
 
-    if (packet->length() > *quota) {
-      m_uploadThrottle->node_used(&m_uploadNode, used);
-      return false;
-    }
-
     queue.pop_front();
 
     try {
@@ -829,9 +812,6 @@ DhtServer::process_queue(packet_queue& queue, uint32_t* quota) {
       if (written == -1)
         throw network_error();
 
-      used += written;
-      *quota -= written;
-
       if (static_cast<unsigned int>(written) != packet->length())
         throw network_error();
 
@@ -854,9 +834,6 @@ DhtServer::process_queue(packet_queue& queue, uint32_t* quota) {
         packet->transaction()->set_packet(NULL);
     }
   }
-
-  m_uploadThrottle->node_used(&m_uploadNode, used);
-  return true;
 }
 
 void
@@ -864,21 +841,11 @@ DhtServer::event_write() {
   if (m_highQueue.empty() && m_lowQueue.empty())
     throw internal_error("DhtServer::event_write called but both write queues are empty.");
 
-  if (!m_uploadThrottle->is_throttled(&m_uploadNode))
-    throw internal_error("DhtServer::event_write called while not in throttle list.");
+  process_queue(m_highQueue);
+  process_queue(m_lowQueue);
 
-  uint32_t quota = m_uploadThrottle->node_quota(&m_uploadNode);
-
-  if (quota == 0 || !process_queue(m_highQueue, &quota) || !process_queue(m_lowQueue, &quota)) {
+  if (m_highQueue.empty() && m_lowQueue.empty())
     this_thread::poll()->remove_write(this);
-    m_uploadThrottle->node_deactivate(&m_uploadNode);
-    return;
-  }
-
-  if (m_highQueue.empty() && m_lowQueue.empty()) {
-    this_thread::poll()->remove_write(this);
-    m_uploadThrottle->erase(&m_uploadNode);
-  }
 }
 
 void
@@ -887,10 +854,8 @@ DhtServer::event_error() {
 
 void
 DhtServer::start_write() {
-  if ((!m_highQueue.empty() || !m_lowQueue.empty()) && !m_uploadThrottle->is_throttled(&m_uploadNode)) {
-    m_uploadThrottle->insert(&m_uploadNode);
+  if (!m_highQueue.empty() || !m_lowQueue.empty())
     this_thread::poll()->insert_write(this);
-  }
 
   if (!m_task_timeout.is_scheduled() && !m_transactions.empty())
     this_thread::scheduler()->wait_for_ceil_seconds(&m_task_timeout, 5s);
